program Demo;

uses
  windows, messages,
  enZGLine;

var
  mode: TMode = (ResX: 1280;
                 ResY: 1024;
                 bpp: 32;
                 full: True;
                 Freq: -1;
                 AntiAliasing: True;
                 Samples: -1;
                 Anisotropy: true;
                 v_sync: false);

  M_tor: TGLModel;
  T_tex: TGLTexture;
  T_nor: TGLTexture;
  Angle: single;
  S_Shader: TGLSLProgram;
  L_I: integer;
  T_T: integer;
  T_N: integer;

  F_mode: byte = 0;
  Simple: boolean = false;

function IntToStr(i: integer): string;
begin
  str(i,result);
end;

procedure glInit;
begin
  Angle := 0;
  fsys.AddPacksDir('Data');

  T_nor := tex.LoadTexture('Data\nor.tga',true,true);  
  T_tex := tex.LoadTexture('Data\tex.tga',true,true);
  M_tor := mdl.LoadModel('Data\Torus.zmm',-1,true);

  S_Shader := vfp.LoadProgram('Data\Shader');
  L_I := vfp.GetUniformLocation(S_Shader,'LightPos');
  T_T := vfp.GetUniformLocation(S_Shader,'T_tex');
  T_N := vfp.GetUniformLocation(S_Shader,'T_nor');
end;

procedure glFree;
begin

end;

procedure glMsg(m: UINT; w: WPARAM; l: LPARAM);
begin
  case m of
    WM_KEYUP:
    begin
      case w of
        VK_TAB:
        begin
          F_mode := (F_Mode + 1) mod 4;
        end;
        VK_F1: tex.ScreenShot('screen.bmp');
        VK_F2: tex.ScreenShot('screen.tga');
        VK_F3:
        begin
          mode.full := not mode.full;
          wnd.SetMode(mode);
        end;
        VK_F4: Simple := not Simple;
      end;
    end;
  end;
end;

procedure glUpdate;
begin
  if inp.KeyDown(VK_ESCAPE) then eng.Quit;
  angle := angle + 1;
end;

procedure glRender;
begin
  ogl.Clear(true,true);
  tex.EnableTextures2D;
  ogl.Set3D(0.1,100.0,90,1.333);
  tex.FilterGroup(F_mode);
  tex.EnableTextures2D;
  tex.Enable(T_nor);
  tex.Enable(T_tex);

  if not simple then
  begin
    vfp.SetUniform1i(S_Shader,T_T,T_tex);
    vfp.SetUniform1i(S_Shader,T_N,T_nor);
    vfp.SetUniform3f(S_Shader,L_I,-15,0,15);
    vfp.Enable(S_Shader);
  end;

  ogl.Translate(0,0,-10);
  ogl.Rotate(Angle,-1,2,1);
  ogl.Scale(0.05,0.05,0.05);

  mdl.Render(M_tor,0);

  tex.DisableTextures2D;

  if not simple then vfp.Disable;

  ogl.Set2D(800,600);
  ogl.glTextOut('FPS: '+IntToStr(eng.FPS),10,10,ogl.DebugFont);
  case F_mode of
    TF_NONE      : ogl.glTextOut('[TAB]Filter : None',10,30,ogl.DebugFont);
    TF_BILINEAR  : ogl.glTextOut('[TAB]Filter : Bilinear',10,30,ogl.DebugFont);
    TF_TRILINEAR : ogl.glTextOut('[TAB]Filter : Trilinear',10,30,ogl.DebugFont);
    TF_ANISOTROPY: ogl.glTextOut('[TAB]Filter : Anisotropy',10,30,ogl.DebugFont);
  end;
  ogl.glTextOut('[F1]ScreenShot to "screen.bmp"',10,50,ogl.DebugFont);
  ogl.glTextOut('[F2]ScreenShot to "screen.tga"',10,70,ogl.DebugFont);
  if wnd.GetMode.Full
    then ogl.glTextOut('[F3]Fullscreen',10,90,ogl.DebugFont)
    else ogl.glTextOut('[F3]Windowed',10,90,ogl.DebugFont);
  if wnd.GetMode.Samples = -1
    then ogl.glTextOut('AntiAliasing: 4x',10,110,ogl.DebugFont)
    else ogl.glTextOut('AntiAliasing: '+IntToStr(wnd.GetMode.Samples)+'x',10,110,ogl.DebugFont);
  ogl.glTextOut('[F4]Use shader: '+IntToStr(Integer(not simple)),10,130,ogl.DebugFont);
  ogl.glTextOut('ResX: '+IntToStr(wnd.GetMode.ResX),10,150,ogl.DebugFont);
  ogl.glTextOut('ResY: '+IntToStr(wnd.GetMode.ResY),10,170,ogl.DebugFont);  

  ogl.Swap;
end;

begin
  InitENZGLINE;//(mode);

  eng.SetProc(PROC_INIT,   @glInit);
  eng.SetProc(PROC_FREE,   @glFree);
  eng.SetProc(PROC_UPDATE, @glUpdate);
  eng.SetProc(PROC_RENDER, @glRender);
  eng.SetProc(PROC_MSG,    @glMsg);

  eng.Loop(50);
end.
 